// ===========================================================================
// CONTENT  : INTERFACE IExplorableElementVisiting
// AUTHOR   : M.Duchrow
// VERSION  : 1.0 - 08/01/2007
// HISTORY  :
//  08/01/2007  mdu  CREATED
//
// Copyright (c) 2007, by Manfred Duchrow. All rights reserved.
// ===========================================================================
package org.pfsw.odem ;

// ===========================================================================
// IMPORTS
// ===========================================================================

/**
 * Specifies all methods that are necessary to fully provide the "Vistor Pattern"
 * in conjunction with the IxxxVisitor interfaces defined here in the same package. 
 *
 * @author M.Duchrow
 * @version 1.0
 */
public interface IExplorableElementVisiting
{ 
	/**
	 * Implementation of the visitor pattern (GOF) for all derived interfaces
	 * if IExplorableElement.
	 * 
	 * @param visitor The implementor of the vistor interface to be called by this method
	 * @return true if processing should be continued, otherwise false
	 */
	public boolean accept( IExplorableElementVisitor visitor ) ;
	
	// -------------------------------------------------------------------------
	
	/**
	 * Implementation of the visitor pattern (GOF) for all derived interfaces
	 * if IExplorableElement. It iterates (recursivly) over all contained IType
	 * objects and invokes the visitors callback method using the types as parameter.
	 * 
	 * @param visitor The implementor of the vistor interface to be called by this method
	 * @return true if processing should be continued, otherwise false
	 */
	public boolean acceptTypes( ITypesVisitor visitor ) ;
	
	// -------------------------------------------------------------------------
	  
} // interface IExplorableElementVisiting