// ===========================================================================
// CONTENT  : ABSTRACT CLASS TypeDependency
// AUTHOR   : M.Duchrow
// VERSION  : 1.0 - 09/02/2007
// HISTORY  :
//  09/02/2007  mdu  CREATED
//
// Copyright (c) 2007, by M.Duchrow. All rights reserved.
// ===========================================================================
package org.pfsw.odem ;

// ===========================================================================
// IMPORTS
// ===========================================================================

/**
 * This abstract dependency class implements some convenience methods to  
 * find out the kind of dependency between two types.
 *
 * @author M.Duchrow
 * @version 1.0
 */
abstract public class TypeDependency<S extends IType, T extends IType> implements IDependency<S,T>
{
  // =========================================================================
  // CONSTANTS
  // =========================================================================

  // =========================================================================
  // INSTANCE VARIABLES
  // =========================================================================

  // =========================================================================
  // CONSTRUCTORS
  // =========================================================================
  /**
   * Initialize the new instance with default values.
   */
  public TypeDependency()
  {
    super() ;
  } // TypeDependency() 

  // =========================================================================
  // PUBLIC INSTANCE METHODS
  // =========================================================================
  /**
   * Returns true if the target type of this dependency is used in one of 
   * the source type's methods.
   */
  public boolean isReferencedInMethodSignature() 
	{
		return this.isReturnTypeOfMethod() || this.isParamTypeInMethod() ;
	} // isReferencedInMethodSignature() 
	
	// -------------------------------------------------------------------------

  /**
   * Returns true if the source type of this dependency either extends or
   * implements the target type.
   */
  public boolean isDerived() 
  {
  	return this.isExtension() || this.isImplementation() ;
  } // isDerived() 
  
  // -------------------------------------------------------------------------
  
  /**
   * Returns true if the target type of this dependency is the type of a
   * method parameter in one of the source type's methods.
   */
  public boolean isParamTypeInMethod() 
  {
  	return DependencyClassification.METHOD_PARAM == this.getDependencyClassification() ;
  } // isParamTypeInMethod() 
  
  // -------------------------------------------------------------------------
  
  /**
   * Returns true if the target type of this dependency is the type of a
   * method parameter in one of the source type's methods.
   */
  public boolean isReturnTypeOfMethod() 
  {
  	return DependencyClassification.METHOD_RESULT == this.getDependencyClassification() ;
  } // isReturnTypeOfMethod() 
  
  // -------------------------------------------------------------------------
  
  /**
   * Returns true if the target type of this dependency is the type of a
   * field (i.e. instance variable or static variable) in source type.
   */
  public boolean isTypeOfField() 
  {
  	return DependencyClassification.FIELD_TYPE == this.getDependencyClassification() ;
  } // isTypeOfField() 
  
  // -------------------------------------------------------------------------
  
  /**
   * Returns true if the target type of this dependency is the type of a
   * local variable (i.e. a field declared locally inside a method) in source type.
   */
  public boolean isTypeOfLocalVariable() 
  {
  	return DependencyClassification.LOCAL_FIELD_TYPE == this.getDependencyClassification() ;
  } // isTypeOfLocalVariable() 
  
  // -------------------------------------------------------------------------
  
  /**
   * Returns true if the source type of this dependency extends the target type.
   */
  public boolean isExtension() 
  {
  	return DependencyClassification.EXTENSION == this.getDependencyClassification() ;
  } // isExtension() 
  
  // -------------------------------------------------------------------------
  
  /**
   * Returns true if the source type of this dependency implements the 
   * target (interface) type.
   */
  public boolean isImplementation() 
  {
  	return DependencyClassification.IMPLEMENTATION == this.getDependencyClassification() ;
  } // isImplementation() 
  
  // -------------------------------------------------------------------------
  
  // =========================================================================
  // PROTECTED INSTANCE METHODS
  // =========================================================================

} // class TypeDependency 
